/////////////////////////////////////////////////////////////////////////////
//  File:       BinTree.h
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _BINTREE_H
#define _BINTREE_H

#define LEFT 0
#define RIGHT 1

#include "node.h"
#include "nodeentities.h"
#include "bintreeformat.h"
#include "undo.h"

// ==============================================================
// binary tree datastructure
class CBinTree : public CObject, public CUndo
{
public:
	DECLARE_SERIAL(CBinTree)
	CBinTree();
	virtual ~CBinTree();

	const void operator=(const CBinTree& BinTree);
	virtual void Serialize(CArchive& ar);

protected:
	// the rootnode of the tree
	CNode* m_pRootNode;
	// the selected node
	CNode* m_pSelectNode;
	// the node-selection type (one of NS_*, see node.h)
	DWORD m_dwSelectType;
	// the name of the tree
	CString m_strName;

public:
	// creates a tree with a root node of type strType
	CNode* CreateTree(CString strType);
	// returns the root node
	CNode* GetRootNode();
	// sets the root node
	void SetRootNode(CNode* pNode);

	// Sets and Gets the Name of the Bintree
	void SetName(CString strName);
	CString GetName();
	
	// Select the node pNode with nodeselectiontype dwSelectType
	void SelectNode(CNode* pNode, DWORD dwSelectType);
	// Get the selected node
	CNode* GetSelectedNode();

	// Set and get the nodeselectiontype for the tree
	void SetNodeSelectionType(DWORD dwSelectType);
	DWORD GetNodeSelectionType();

	// reset the tree and delete all nodes
	void ResetTree();

	// get the parent of node pNode
	CNode* GetParent(CNode* pNode);
	
	//creates a node of type strType
	CNode* CreateNode(CString strType);
	
	// some handy functions for rearrangement of the tree
	CNode* ReplaceNode(CNode* pDestNode, CString strType);
	void ReplaceNode(CNode* pDestNode, CNode* pSrcNode);
	CNode* ReplaceSubTree(CNode* pDestNode, CString strType);
	CNode* InsertNode(CNode* pParent, CNode* pChild, CString strType, DWORD dwInsert);

	// draw the tree on a dc
	void DrawTree(CDC* pDC, CRect rect);
	
	// get the destination rect of the tree
	CRect GetRect(CDC* pDC);
	
	// get a node that draws itself into a rect (resultRect) that contains point 
	CNode* GetNodeFromPoint(CDC* pDC, CRect& resultRect, CPoint point);
	
	// reformat the tree with a special formatter
	void ReformatNodes(CBinTreeFormat* pFormat=NULL);
	
	// sorting the tree in associative order (currently not used)
	void SortPreOrder(CNode* pOldNode);
	CNode* SwapLeftChildNode(CNode* pNewNode, CNode* pOldNode);
	CNode* SwapRightChildNode(CNode* pNewNode, CNode* pOldNode);

	// write the binary-tree content into a dib
	BOOL WriteDIB(CString strFile);
	// write the binary-tree content into a dib
	BOOL WriteWMF(CString strFile);
	// print the binary-tree content into a DC
	BOOL PrintDDB(CDC* pDC);
	// write the binary-tree content into a CString
	CString WriteFormula(int nLanguageType);

protected:
	// reset the tree and delete all nodes (from CUndo)
	virtual void DeleteContents();
	// delete a node
	void DeleteNode(CNode* pNode);
	// clone a node with or without its subtree
	CNode* CloneNode(CNode* pNode, BOOL bCloneSubTree=TRUE);
};
#endif
