// FormulaDropTarget.cpp : implementation file
//
// MFC Formula Control - Drag/Drop target implementation
//
// CFormulaDropTarget is an OLE drop target for CFormulaCtrl. All it does
// is handle the drag and drop windows messages and pass them
// directly onto the Formula control.
//
// Written by Chris Maunder <cmaunder@mail.com>
// Copyright (c) 1998-2001. All Rights Reserved.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. 
//
// An email letting me know how you are using it would be nice as well. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FormulaCtrl.h"

#include "FormulaDropTarget.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFormulaDropTarget

CFormulaDropTarget::CFormulaDropTarget()
{
    m_pFormulaCtrl = NULL;
    m_bRegistered = FALSE;
}

CFormulaDropTarget::~CFormulaDropTarget()
{
}

// Overloaded Register() function performs the normal COleDropTarget::Register
// but also serves to connect this COleDropTarget with the parent Formula control,
// where all drop messages will ultimately be forwarded.
BOOL CFormulaDropTarget::Register(CFormulaCtrl *pFormulaCtrl)
{
    if (m_bRegistered)
        return FALSE;

    // Stop re-entry problems
    static BOOL bInProcedure = FALSE;
    if (bInProcedure)
        return FALSE;
    bInProcedure = TRUE;

    ASSERT(pFormulaCtrl->IsKindOf(RUNTIME_CLASS(CFormulaCtrl)));
    ASSERT(pFormulaCtrl);

    if (!pFormulaCtrl || !pFormulaCtrl->IsKindOf(RUNTIME_CLASS(CFormulaCtrl)))
    {
        bInProcedure = FALSE;
        return FALSE;
    }

    m_pFormulaCtrl = pFormulaCtrl;

    m_bRegistered = COleDropTarget::Register(pFormulaCtrl);

    bInProcedure = FALSE;
    return m_bRegistered;
}

void CFormulaDropTarget::Revoke()
{
    m_bRegistered = FALSE;
    COleDropTarget::Revoke();
}

BEGIN_MESSAGE_MAP(CFormulaDropTarget, COleDropTarget)
    //{{AFX_MSG_MAP(CFormulaDropTarget)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFormulaDropTarget message handlers

DROPEFFECT CFormulaDropTarget::OnDragScroll(CWnd* pWnd, DWORD dwKeyState, CPoint /*point*/)
{
    if (pWnd->GetSafeHwnd() == m_pFormulaCtrl->GetSafeHwnd())
    {
        if (dwKeyState & MK_CONTROL)
            return DROPEFFECT_COPY;
        else
            return DROPEFFECT_MOVE;
    } 
	else
        return DROPEFFECT_NONE;
}

DROPEFFECT CFormulaDropTarget::OnDragEnter(CWnd* pWnd, COleDataObject* pDataObject, 
                                        DWORD dwKeyState, CPoint point)
{
    ASSERT(m_pFormulaCtrl);

    if (pWnd->GetSafeHwnd() == m_pFormulaCtrl->GetSafeHwnd())
        return m_pFormulaCtrl->OnDragEnter(pDataObject, dwKeyState, point);
    else
        return DROPEFFECT_NONE;
}

void CFormulaDropTarget::OnDragLeave(CWnd* pWnd)
{
    ASSERT(m_pFormulaCtrl);

    if (pWnd->GetSafeHwnd() == m_pFormulaCtrl->GetSafeHwnd())
        m_pFormulaCtrl->OnDragLeave();
}

DROPEFFECT CFormulaDropTarget::OnDragOver(CWnd* pWnd, COleDataObject* pDataObject, 
                                       DWORD dwKeyState, CPoint point)
{
    ASSERT(m_pFormulaCtrl);

    if (pWnd->GetSafeHwnd() == m_pFormulaCtrl->GetSafeHwnd())
        return m_pFormulaCtrl->OnDragOver(pDataObject, dwKeyState, point);
    else
        return DROPEFFECT_NONE;
}

BOOL CFormulaDropTarget::OnDrop(CWnd* pWnd, COleDataObject* pDataObject,
                             DROPEFFECT dropEffect, CPoint point)
{
    TRACE(_T("In CFormulaDropTarget::OnDrop\n"));
    ASSERT(m_pFormulaCtrl);

    if (pWnd->GetSafeHwnd() == m_pFormulaCtrl->GetSafeHwnd())
        return m_pFormulaCtrl->OnDrop(pDataObject, dropEffect, point);
    else
        return FALSE;
}
